type RecursivePartial<T> = {
    [P in keyof T]?: RecursivePartial<T[P]>;
};
type QueryInstructions = {
    with: Record<string, number | string | null | any>;
    where: any;
    to: Record<string, number | string | null | any>;
    including: string[];
    excluding: string[];
    orderedBy: {
        ascending: string[];
        descending: string[];
    };
    in: string;
    after: number;
    limitedTo: number;
};
type QueryType = 'get' | 'set' | 'drop' | 'create' | 'count';
type QuerySchema = Record<string, QueryInstructions>;
type Query = Record<QueryType, QuerySchema>;
type QueryResultList<T> = T[] & {
    moreAfter?: number;
};
type QueryResult<T> = T extends never ? null | number | unknown | QueryResultList<unknown> : T extends Array<infer T> ? QueryResultList<T> : T;
type ReadableValue = File | ReadableStream;
type Reference = {
    key: string;
} | Media;
interface Readable {
    /** Index and type of the Query the `Readable` was found in. */
    query: {
        index: number;
        type: 'set' | 'create';
    };
    /** Schema to update. */
    schema: string;
    /** Name of the field to set the value to. */
    field: string;
    /** Stream containing the value to be set. */
    value: ReadableValue;
    /** Content-Type of the streamed value. */
    contentType: string;
}
interface StoredReadable extends Readable {
    /** Reference to the stored value or Media object. */
    reference: Reference;
}
type MediaMetadata = {
    /** Size of the image or video in bytes. */
    size: number | null;
    /** MIME type of the image or video. */
    type: string | null;
    /** Width of the image or video in pixels. */
    width: number | null;
    /** Height of the image or video in pixels. */
    height: number | null;
};
type MediaPlaceholder = {
    /** Base64 representation of the generated placeholder. */
    base64: string | null;
};
type StoredMedia = {
    /** Key under which the media item is stored inside RONIN. */
    key: string;
    /** Object containing various metadata of the stored media item. */
    meta: MediaMetadata;
    /** Object containing different versions of the generated placeholder. */
    placeholder: MediaPlaceholder | null;
};
type Media = StoredMedia & {
    /** URL under which the stored image or video can be accessed from the client.  */
    src: string;
};
/**
 * @see https://twitter.com/mattpocockuk/status/1622730173446557697
 */
type Prettify<T> = {
    [K in keyof T]: T[K];
} & {};

type Fetcher = typeof fetch;

type ReadableProcessingOptions = Partial<{
    /**
     * Allows for specifying a custom endpoint `Readable`s will be uploaded to.
     *
     * This endpoint has to respond with a JSON object of the following format:
     *
     * ```js
     * const res = (...) => {
     *   // Where the value of `id` has to be a valid UUID v4.
     *   key: "35bc40a1-eaec-4163-8259-37c6b9723740"
     * }
     * ```
     */
    endpoint: string;
    /** Allows for specifying a custom HTTP request method. Defaults to `PUT`. */
    method: string;
    /** Allows for specifying a custom `fetch` function used to make network requests. */
    fetcher?: Fetcher;
}>;
/**
 * ## 🚧 For internal use only! 🚧
 *
 * Store `Readable`s from given queries and populate the queries with the respective references.
 * @param queries `Query` array which may or may not include `Readable`s.
 * @param options `ReadableProcessingOptions` object containing options passed on to the internal handling of `Readable`s.
 * @returns Queries populated with the references to each `Readable` they might have initially contained.
 */
declare const processReadables: (queries: Query[], token: string, options?: ReadableProcessingOptions) => Promise<Query[]>;

type QueryProcessingOptions = {
    /**
     * Allows for specifying a custom HTTP endpoint that the provided queries
     * should be submitted to.
     */
    endpoint?: string;
    /**
     * Allows for specifying a custom replacement for the native JavaScript
     * `fetch` function used to make network requests for submitting queries.
     */
    fetcher?: Fetcher;
    /**
     * Allows for specifying a custom source database other than the one
     * associated with the provided app token.
     */
    database?: string;
};

declare namespace RONIN {
    interface RoninMetadata {
        createdAt: Date;
        createdBy: `rec_${string}`;
        locked: boolean;
        status: 'draft' | 'published' | 'archived';
        updatedAt: Date;
        updatedBy: `rec_${string}`;
    }
    export interface RoninRecord {
        id: `rec_${string}`;
        ronin: RoninMetadata;
    }
    interface AllSubFilters<T> {
        contains: string | T;
        endsWith: string | T;
        is: T;
        isGreaterThan: number | Date;
        isLessThan: number | Date;
        isNot: T;
        startsWith: string | T;
    }
    type SubFilter<T> = T extends number ? Prettify<Pick<AllSubFilters<T>, 'isGreaterThan' | 'isLessThan' | 'is'>> : T extends string ? Prettify<Pick<AllSubFilters<T>, 'contains' | 'endsWith' | 'is' | 'isNot' | 'startsWith'>> : T extends boolean ? Prettify<Pick<AllSubFilters<T>, 'is'>> : T extends Date ? Prettify<Pick<AllSubFilters<T>, 'isGreaterThan' | 'isLessThan' | 'is'>> : never;
    type Where<TSchema extends Record<string, unknown>> = {
        [TKey in keyof TSchema]: SubFilter<TSchema[TKey]>;
    };
    type With<TSchema extends Record<string, unknown>> = {
        [TKey in keyof TSchema]: TSchema[TKey];
    };
    type AllFields = 'createdAt' | 'updatedAt' | 'name' | 'slug' | 'amount';
    type RecordFields = 'views' | 'accounts' | 'spaces';
    interface OrderedBy {
        /** Order the resulting records in descending order using a specific field. */
        descending: Array<AllFields>;
        /** Order the resulting records in ascending order using a specific field. */
        ascending: Array<AllFields>;
    }
    type To<TSchema extends Record<string, unknown>> = With<TSchema>;
    type Variants = 'english' | 'german' | 'japanese';
    export interface Filter<TSchema extends Record<string, unknown> & RoninRecord> {
        /** Perform specific matching actions (such as "is", "contains", or "startsWith") on the fields of a record. */
        where: Partial<Where<TSchema>>;
        /** Perform "is" matching actions on the fields of a record. */
        with: Partial<With<TSchema>>;
        /** Order the resulting records using a specific field. */
        orderedBy: Partial<OrderedBy>;
        /** Resolve records in a specific variant. */
        in: Partial<Variants>;
        /** Resolve the records associated with certain fields of type "Record". */
        including: Array<Partial<RecordFields>>;
    }
    export interface FilterAndMutate<TSchema extends Record<string, unknown> & RoninRecord> extends Filter<TSchema> {
        /** Perform mutations on the fields of a record or add new fields to it. */
        to: Partial<To<TSchema>>;
    }
    export interface Counter extends Record<string, Filter<Record<string, unknown> & RoninRecord>> {
    }
    export interface Creator extends Record<string, Filter<Record<string, unknown> & RoninRecord>> {
    }
    export interface Dropper extends Record<string, Filter<Record<string, unknown> & RoninRecord>> {
    }
    export interface Getter extends Record<string, Filter<Record<string, unknown> & RoninRecord>> {
    }
    export interface Setter extends Record<string, FilterAndMutate<Record<string, unknown> & RoninRecord>> {
    }
    export {};
}

interface RunScratchpadOptions {
    /**
     * Allows for specifying custom data hooks that are executed for every query
     * of a particular schema.
     */
    hooks?: Record<string, unknown>;
    /**
     * Allows for passing a custom app token. In a Node.js-based environment,
     * `process.env.RONIN_TOKEN` will be used by default.
     */
    token?: string;
    /**
     * ## 🚧 For internal use only! 🚧
     *
     * Allows for specifying custom options used for processing readables.
     */
    readableProcessingOptions?: ReadableProcessingOptions;
    /**
     * ## 🚧 For internal use only! 🚧
     *
     * Allows for specifying custom options used for processing queries.
     */
    queryProcessingOptions?: QueryProcessingOptions;
}
interface ScratchPadActions {
    /**
     * Retrieve existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Simple Example
     * get.account.with.handle = "juri";
     *
     * // Advanced Example
     * get.accounts = {
     *   where: {
     *     role: {
     *       is: "ADMIN" // 👈🏻 Match specific fields
     *     }
     *   }
     *   orderedBy: {
     *     ascending: ["createdAt"]
     *   },
     *   include: ["roles"], // 👈🏻 Resolve nested records
     * }
     * ```
     */
    get: RONIN.Getter;
    /**
     * Modify existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * set.account = {
     *   with: {
     *     handle: "juri",
     *   },
     *   to: {
     *     verified: true,
     *   },
     * }
     * ```
     */
    set: RONIN.Setter;
    /**
     * Create a new record.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * create.account.with = {
     *   handle: "juri",
     *   verified: false,
     * }
     * ```
     */
    create: RONIN.Creator;
    /**
     * Drop existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * drop.accounts.where.handle.isNot = "juri",
     * ```
     */
    drop: RONIN.Dropper;
    /**
     * Count matching records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example:
     * count.accounts.where.verified.is = true;
     * ```
     */
    count: RONIN.Counter;
}
type ScratchPad = (actions: ScratchPadActions) => void;

type BeforeHook = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean) => Promise<RecursivePartial<QueryInstructions>>;
type DuringHook<T = unknown> = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean) => Promise<T>;
type AfterHook<T = unknown> = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean, result: QueryResult<T>) => Promise<void>;

/**
 * Transform a given `ScratchPad` into `Queries` (array) which can be passed to RONIN.
 * @param scratchPad `ScratchPad` to get executable `Queries` from.
 * @returns `Queries` which will be passed to RONIN.
 */
declare const getQueriesFromScratchPad: (scratchPad: ScratchPad) => Query[];

declare const runQueries: <T>(queries: Query[], options?: RunScratchpadOptions) => Promise<QueryResult<T>[]>;

type ScratchPadFactory = <T>(scratchPad: ScratchPad) => Promise<QueryResult<T>[]>;
declare function handleScratchPadOrConfig(config?: RunScratchpadOptions): ScratchPadFactory;
declare function handleScratchPadOrConfig<T extends object | Array<unknown> = [unknown]>(scratchPad: ScratchPad, config?: RunScratchpadOptions): Promise<T extends [] ? [] : T extends [infer First, ...infer Rest] ? (Rest extends unknown ? T : Array<First>) : [T]>;

export { AfterHook, BeforeHook, DuringHook, Query, QueryResult, RONIN, Readable, ReadableValue, RunScratchpadOptions, ScratchPad, ScratchPadActions, StoredReadable, handleScratchPadOrConfig as default, getQueriesFromScratchPad, processReadables, runQueries };
